import argparse
from twitch_to_clip.dl_and_create import dlAndCreate, getIdFromUrl

parser = argparse.ArgumentParser()

# TODO customformated outputs
parser.add_argument("urls", nargs="+", type=str, help="Twitch clip or highlight url/s")
parser.add_argument(
    "-o", "--output", action="store", dest="output", help="Output file name"
)
parser.add_argument(
    "--twitchdl", help="Arguments to pass to twitchdl download. Encase in quotes."
)
parser.add_argument(
    "--ffmpeg", help="Optional ffmpeg command to apply to videos. Encase in quotes."
)

# ! Script params
args = parser.parse_args()
urls = args.urls
output = args.output
twitchdlArgsString = args.twitchdl
ffmpegArgsString = args.ffmpeg


def main():
    print("urls:", urls)
    for url in urls:
        print("url:", url)
        # * Create videos from twitch
        (videoName, title, user, game) = dlAndCreate(
            getIdFromUrl(url),
            outputName=output,
            twitchdlArgs=convertToArgsArray(twitchdlArgsString),
            ffmpegArgs=ffmpegArgsString,
        )

        print("Output Name:", videoName)

    print("---Done---")


def convertToArgsArray(args: str):
    return args.split(" ") if args else []



from twitch_to_clip.ffmpeg_funcs import (
    createVerticalVideo,
    addBlackToEnd,
    getVidLength,
    customFinalCommand,
)


def createShort(clipName: str, outputName: str, ffmpegArgs: str = None):
    createVerticalVideo(clipName, outputName)

    # videoLength = getVidLength(outputName)
    # if videoLength < 15:
    #     addBlackToEnd(outputName, outputName, 15 - videoLength)

    if ffmpegArgs:
        print("\n---applying custom ffmpeg arguments---\n")
        customFinalCommand(outputName, ffmpegArgs)



from twitch_to_clip.create_short import createShort
from twitch_to_clip.dl_twitch import downloadTwitch


def dlAndCreate(
    id: str, outputName: str = None, twitchdlArgs: list = None, ffmpegArgs: list = None
):
    print("\n---Downloading---\n")
    (title, user, game, outputVidName) = downloadTwitch(
        id, outputName=outputName, twitchdlArgs=twitchdlArgs
    )

    print("\n---Creating Video---\n")
    createShort(f"raw_{outputVidName}.mp4", f"{outputVidName}.mp4", ffmpegArgs)

    return (f"{outputVidName}.mp4", title, user, game)


def isVideo(url: str):
    return "/videos/" in url


def getIdFromUrl(url: str):
    if "/clip/" not in url and "/videos/" not in url:
        return url

    return (url.split("/videos/") if isVideo(url) else url.split("/clip/"))[1].split(
        "?"
    )[0]



import shutil
from twitchdl import twitch, console
from twitchdl.commands import download
import string

parser = console.get_parser()


def downloadTwitch(
    tId: str, outputName: str = None, twitchdlArgs: list = None, video: bool = False
):
    """
    Downloads a twitch clip or video and returns the title, user, game, and output name.
    @tId: The id of the twitch clip or video.
    @outputName: The name of the output file.
    @twitchdlArgs: A list of arguments to pass to twitchdl, in argparse list form.
    @video: Whether the id is for a video or not.
    """

    vidInfo = twitch.get_clip(tId) if not video else twitch.get_video(tId)
    if not vidInfo:
        print("No video info")
        return

    if vidInfo["durationSeconds"] > 600:
        print("Video too long")
        return

    title = vidInfo["title"]
    user = vidInfo["broadcaster"]["login"] if not video else vidInfo["creator"]["login"]
    game = vidInfo["game"]["name"]

    # Need to remove punctuation from title for ffmpeg
    outputVidName = (
        title.translate(str.maketrans("", "", string.punctuation))
        if not outputName
        else outputName
    )

    args = parser.parse_args(
        ["download"]
        + [tId]
        + removeArgs(twitchdlArgs[:], ["-f", "--format", "-o", "--output"])
        + ["-f", "mp4"]
        + ["-o", "temp.{format}"]
        + (
            ["-q", "source"]
            if not any(flag in twitchdlArgs for flag in ["-q", "--quality"])
            else []
        )
    )
    print(args)
    download(args)
    shutil.move(f"temp.mp4", f"raw_{outputVidName}.mp4")

    return (title, user, game, outputVidName)


def removeArgs(args: list, argsToRemove: list):
    for arg in argsToRemove:
        if arg in args:
            index = args.index(arg)
            del args[index : index + 2]

    return args


# flags = ' '.join(f'{key} {value}' for key, value in flags.items() if value)
# downloadCommand = f"twitch-dl download {tId} {flags}"
# subprocess.call(downloadCommand)



import os
import math
import subprocess
from subprocess import check_output


def createVerticalVideo(clipName: str, outputName: str):
    inputFile = f"{clipName}"
    outputFile = f"{outputName}"

    command = f'ffmpeg -i "{inputFile}" -lavfi "[0:v]scale=256/81*iw:256/81*ih,boxblur=luma_radius=min(h\,w)/40:luma_power=3:chroma_radius=min(cw\,ch)/40:chroma_power=1[bg];[bg][0:v]overlay=(W-w)/2:(H-h)/2,setsar=1,crop=w=iw*81/256" -c:a copy "{outputFile}"'
    print("command", command)
    subprocess.run(command, shell=True)


def addBlackToEnd(clipName: str, outputName: str, time: str):
    inputFile = f"{clipName}"
    outputFile = f"{outputName}"

    command = f'ffmpeg -i "{inputFile}" -filter_complex "tpad=stop_duration={time}" "temp_{outputFile}"'
    print("command", command)
    subprocess.run(command, shell=True)

    os.replace(f"temp_{outputName}", f"{clipName}")


def getVidLength(clipName: str):
    inputFile = f"{clipName}"

    command = (
        f'ffprobe -i "{inputFile}" -show_entries format=duration -v quiet -of csv="p=0"'
    )
    print("command", command)
    return math.floor(float(str(check_output(command, shell=True).decode("utf-8"))))


def customFinalCommand(clipName: str, command: str):
    """
    clipName: Name of the clip to be edited
    command: ffmpeg command e.g., "-af highpass=f=100 \"output.mp4\""
    """
    inputFile = f"{clipName}"

    command = f'ffmpeg -i "{inputFile}" {command}'
    subprocess.run(command, shell=True)



from setuptools import setup, find_packages

setup(
    name="twitch-to-clip",
    description="Create vertical videos from twitch clips",
    version="0.5.0",
    packages=find_packages(),
    entry_points={
        "console_scripts": [
            "twitch-to-clip = twitch_to_clip.__main__:main",
        ]
    },
)
